from flask import Flask, request, jsonify
import mysql.connector
import json
import time

app = Flask(__name__)

DB_CONFIG = {
    "host": "db",  # Use the service name from docker-compose.yml
    "user": "root",
    "password": "my-secret-pw",
    "database": "testdb"
}

# Retry MySQL Connection
while True:
    try:
        print("Attempting to connect to MySQL...")
        db = mysql.connector.connect(**DB_CONFIG)
        cursor = db.cursor()
        print("Connected to MySQL successfully!")

        # Ensure the 'users' table exists
        cursor.execute("""
            CREATE TABLE IF NOT EXISTS users (
                id INT AUTO_INCREMENT PRIMARY KEY,
                name VARCHAR(255),
                data JSON
            )
        """)
        db.commit()
        break  # Exit loop on success

    except mysql.connector.Error as err:
        print(f"MySQL connection error: {err}")
        print("Retrying in 5 seconds...")
        time.sleep(5)

@app.route('/users', methods=['POST'])
def add_user():
    data = request.get_json()
    name = data.get('name')
    user_data = data.get('data')

    if not name or user_data is None:
        return jsonify({"error": "Invalid request"}), 400

    try:
        db = mysql.connector.connect(**DB_CONFIG)
        cursor = db.cursor()
        cursor.execute("INSERT INTO users (name, data) VALUES (%s, %s)", (name, json.dumps(user_data)))
        db.commit()
        cursor.close()
        db.close()
        return jsonify({"message": "User added!"}), 201

    except mysql.connector.Error as err:
        return jsonify({"error": str(err)}), 500

@app.route('/users', methods=['GET'])
def get_users():
    try:
        db = mysql.connector.connect(**DB_CONFIG)
        cursor = db.cursor(dictionary=True)  # Fetch data as dictionary
        cursor.execute("SELECT * FROM users")
        users = cursor.fetchall()
        cursor.close()
        db.close()
        return jsonify(users)

    except mysql.connector.Error as err:
        return jsonify({"error": str(err)}), 500

if __name__ == '__main__':
    app.run(host='0.0.0.0', port=5000)
